@@ -10,6 +10,7 @@ passing in distutils.
 Specifically, this module is only meant to be imported in setup.py scripts.
 """
 
+import copy
 import errno
 import inspect
 import io
@@ -79,7 +80,7 @@ def version(project=PROJECT):
     """
     version = None
     try:
-        with io.open(os.path.join(project, '__init__.py'), encoding='utf-8') as f:
+        with io.open(os.path.join(TOPDIR, project, '__init__.py'), encoding='utf-8') as f:
             version = re.search(
                 r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                 f.read(), re.MULTILINE).group(1)
@@ -95,6 +96,21 @@ def version(project=PROJECT):
     return version
 
 
+def readme(project=PROJECT):
+    """Determine a project's long description."""
+    for doc in ('README.rst', 'README'):
+        try:
+            with io.open(os.path.join(TOPDIR, doc), encoding='utf-8') as f:
+                return f.read()
+        except IOError as e:
+            if e.errno == errno.ENOENT:
+                pass
+            else:
+                raise
+
+    return None
+
+
 def get_file_paths(path):
     """Get list of all file paths under a given path."""
     for root, dirs, files in os.walk(path):
@@ -145,6 +161,7 @@ def pkg_config(*packages, **kw):
         sys.exit(1)
 
     for token in tokens:
+        token = token.decode()
         if token[:2] in flag_map:
             kw.setdefault(flag_map.get(token[:2]), []).append(token[2:])
         else:
@@ -202,9 +219,23 @@ class sdist(dst_sdist.sdist):
         dst_sdist.sdist.make_release_tree(self, base_dir, files)
         self.generate_verinfo(base_dir)
 
+    def run(self):
+        build_ext = self.reinitialize_command('build_ext')
+        build_ext.ensure_finalized()
+
+        # generate cython extensions if any exist
+        cython = any(
+            os.path.splitext(f)[1] == '.pyx' for e in
+            build_ext.extensions for f in e.sources)
+        if cython:
+            from Cython.Build import cythonize
+            cythonize(build_ext.extensions)
+
+        dst_sdist.sdist.run(self)
+
 
 class build_py(dst_build_py.build_py):
-    """build_py command wrapper that runs 2to3 for py3 targets."""
+    """build_py command wrapper."""
 
     user_options = dst_build_py.build_py.user_options + \
         [("inplace", "i", "do any source conversions in place")]
@@ -222,6 +253,29 @@ class build_py(dst_build_py.build_py):
             self.build_lib = '.'
         dst_build_py.build_py.finalize_options(self)
 
+    def _run_generate_verinfo(self, rebuilds=None):
+        ver_path = self.get_module_outfile(
+            self.build_lib, (self.package_namespace,), '_verinfo')
+        # this should check mtime...
+        if not os.path.exists(ver_path):
+            from snakeoil.version import get_git_version
+            log.info('generating _verinfo')
+            with open(ver_path, 'w') as f:
+                f.write("version_info=%r" % (get_git_version('.'),))
+            self.byte_compile([ver_path])
+            if rebuilds is not None:
+                rebuilds.append((ver_path, os.lstat(ver_path).st_mtime))
+
+    def run(self):
+        dst_build_py.build_py.run(self)
+
+        if self.generate_verinfo:
+            self._run_generate_verinfo()
+
+
+class build_py2to3(build_py):
+    """build_py command wrapper that runs 2to3 for py3 targets."""
+
     def _compute_rebuilds(self, force=False):
         for base, mod_name, path in self.find_all_modules():
             try:
@@ -244,18 +298,6 @@ class build_py(dst_build_py.build_py):
     def _inner_run(self, rebuilds):
         pass
 
-    def _run_generate_verinfo(self, rebuilds):
-        from snakeoil.version import get_git_version
-        ver_path = self.get_module_outfile(
-            self.build_lib, (self.package_namespace,), '_verinfo')
-        # this should check mtime...
-        if not os.path.exists(ver_path):
-            log.info('generating _verinfo')
-            with open(ver_path, 'w') as f:
-                f.write("version_info=%r" % (get_git_version('.'),))
-            self.byte_compile([ver_path])
-            rebuilds.append((ver_path, os.lstat(ver_path).st_mtime))
-
     def get_py2to3_converter(self, options=None, proc_count=0):
         from lib2to3 import refactor as ref_mod
         from snakeoil.dist import caching_2to3
@@ -285,8 +327,7 @@ class build_py(dst_build_py.build_py):
         py3k_rebuilds = []
         if not self.inplace:
             if is_py3k:
-                py3k_rebuilds = list(self._compute_rebuilds(
-                    self.force))
+                py3k_rebuilds = list(self._compute_rebuilds(self.force))
             dst_build_py.build_py.run(self)
 
         if self.generate_verinfo:
@@ -305,7 +346,7 @@ class build_py(dst_build_py.build_py):
         log.info("completed py3k conversions")
 
 
-class build_py3(build_py):
+class build_py3to2(build_py2to3):
     """build_py command wrapper that runs 3to2 for py2 targets."""
 
     def run(self):
@@ -458,9 +499,55 @@ class build_ext(dst_build_ext.build_ext):
                 if self.default_header_install_dir not in e.include_dirs:
                     e.include_dirs.append(self.default_header_install_dir)
 
+    @staticmethod
+    def determine_ext_lang(ext_path):
+        """Determine file extensions for generated cython extensions."""
+        with open(ext_path) as f:
+            for line in f:
+                line = line.lstrip()
+                if not line:
+                    continue
+                elif line[0] != '#':
+                    return None
+                line = line[1:].lstrip()
+                if line[:10] == 'distutils:':
+                    key, _, value = [s.strip() for s in line[10:].partition('=')]
+                    if key == 'language':
+                        return value
+            else:
+                return None
+
+    def no_cythonize(self):
+        """Determine file paths for generated cython extensions."""
+        extensions = copy.deepcopy(self.extensions)
+        for extension in extensions:
+            sources = []
+            for sfile in extension.sources:
+                path, ext = os.path.splitext(sfile)
+                if ext in ('.pyx', '.py'):
+                    lang = build_ext.determine_ext_lang(sfile)
+                    if lang == 'c++':
+                        ext = '.cpp'
+                    else:
+                        ext = '.c'
+                    sfile = path + ext
+                sources.append(sfile)
+            extension.sources[:] = sources
+        return extensions
+
     def run(self):
         # ensure that the platform checks were performed
         self.run_command('config')
+
+        # only regenerate cython extensions if requested or required
+        use_cython = (
+            os.environ.get('USE_CYTHON', False) or
+            any(not os.path.exists(x) for ext in self.no_cythonize() for x in ext.sources))
+        if use_cython:
+            from Cython.Build import cythonize
+            cythonize(self.extensions)
+
+        self.extensions = self.no_cythonize()
         return dst_build_ext.build_ext.run(self)
 
     def build_extensions(self):
@@ -772,7 +859,7 @@ class PyTest(Command):
         ('match=', 'k', 'run only tests that match the provided expressions'),
     ]
 
-    default_test_dir = os.path.join(PROJECT, 'test')
+    default_test_dir = os.path.join(TOPDIR, PROJECT, 'test')
 
     def initialize_options(self):
         self.pytest_args = ''
