@@ -102,14 +102,28 @@ def native_reflective_hash(attr):
     return __hash__
 
 
-class _native_internal_jit_attr(object):
-
-    """
-    object implementing the descriptor protocol for use in Just In Time access to attributes.
+def _native_internal_jit_attr(
+        func, attr_name, singleton=None,
+        use_cls_setattr=False, use_singleton=True, doc=None):
+    """Object implementing the descriptor protocol for use in Just In Time access to attributes.
 
     Consumers should likely be using the :py:func:`jit_func` line of helper functions
     instead of directly consuming this.
     """
+    doc = getattr(func, '__doc__', None) if doc is None else doc
+    kls = _raw_native_internal_jit_attr
+    class _native_internal_jit_attr(kls):
+        __doc__ = doc
+        __slots__ = ()
+    kls = _native_internal_jit_attr
+
+    return kls(
+        func, attr_name, singleton=singleton, use_cls_setattr=use_cls_setattr,
+        use_singleton=use_singleton)
+
+
+class _raw_native_internal_jit_attr(object):
+    """See _native_internal_jit_attr; this is an implementation detail of that"""
 
     __slots__ = ("storage_attr", "function", "_setter", "singleton", "use_singleton")
 
@@ -593,10 +607,12 @@ def patch(cls, method, external_decorator=None):
     >>> @patch(math, 'ceil')
     >>> def ceil(orig_ceil, n):
     ...   return math.floor(n)
-    >>> n = 0.1
-    >>> assert math.ceil(n) == 0
+    >>> assert math.ceil(0.1) == 0
     """
     def decorator(func):
+        # use the original function wrapper
+        func = getattr(func, '_func', func)
+
         # save the original method
         orig_func = getattr(cls, method)
 
@@ -604,6 +620,9 @@ def patch(cls, method, external_decorator=None):
         def wrapper(*args, **kwargs):
             return func(orig_func, *args, **kwargs)
 
+        # save the original function wrapper
+        wrapper._func = func
+
         if external_decorator is not None:
             wrapper = external_decorator(wrapper)
 
