@@ -670,7 +670,7 @@ snakeoil_readlines_get_mtime(snakeoil_readlines *self)
 	Py_DECREF(ret);
 	if (is_float)
 		return PyFloat_FromDouble(self->mtime + 1e-9 * self->mtime_nsec);
-#if SIZEOF_TIME_T > SIZEOF_LONG
+#ifdef TIME_T_LONGER_THAN_LONG
 	return PyLong_FromLong((Py_LONG_LONG)self->mtime);
 #else
 	return PyInt_FromLong((long)self->mtime);
@@ -777,6 +777,7 @@ snakeoil_closerange(PyObject *self, PyObject *args)
 	fd_dir = dirfd(dir_handle);
 
 	if (fd_dir < 0) {
+		closedir(dir_handle);
 		snakeoil_slow_closerange(from, to);
 		Py_RETURN_NONE;
 	}
